-- region stick

local function FrameIsDependentOnFrame(frame, otherFrame)
	if (frame and otherFrame) then
		if frame == otherFrame then return true end
		local points = frame:GetNumPoints()
		for i = 1, points do
			local parent = select(2, frame:GetPoint(i))
			if FrameIsDependentOnFrame(parent, otherFrame) then
				return true
			end
		end
	end
end

local function CanAttach(frame, otherFrame)
	if not(frame and otherFrame) then
		return
	elseif frame:GetNumPoints() == 0 or otherFrame:GetNumPoints() == 0 then
		return
	elseif frame:GetWidth() == 0 or frame:GetHeight() == 0 or otherFrame:GetWidth() == 0 or otherFrame:GetHeight() == 0 then
		return
	elseif FrameIsDependentOnFrame(otherFrame, frame) then
		return
	end
	return true
end

local function AttachToTop(frame, otherFrame, distLeft, distRight, distCenter, offset)
	frame:ClearAllPoints()
	if distLeft < distCenter and distLeft < distRight then
		frame:SetPoint('BOTTOMLEFT', otherFrame, 'TOPLEFT', 0, offset)
	elseif distRight < distCenter and distRight < distLeft then
		frame:SetPoint('BOTTOMRIGHT', otherFrame, 'TOPRIGHT', 0, offset)
	else
		frame:SetPoint('BOTTOM', otherFrame, 'TOP', 0, offset)
	end
	frame:SetSize(otherFrame:GetWidth(), otherFrame:GetHeight())
end

local function AttachToBottom(frame, otherFrame, distLeft, distRight, distCenter, offset)
	frame:ClearAllPoints()
	if distLeft < distCenter and distLeft < distRight then
		frame:SetPoint('TOPLEFT', otherFrame, 'BOTTOMLEFT', 0, -offset)
	elseif distRight < distCenter and distRight < distLeft then
		frame:SetPoint('TOPRIGHT', otherFrame, 'BOTTOMRIGHT', 0, -offset)
	else
		frame:SetPoint('TOP', otherFrame, 'BOTTOM', 0, -offset)
	end
	frame:SetSize(otherFrame:GetWidth(), otherFrame:GetHeight())
end

local function AttachToLeft(frame, otherFrame, distTop, distBottom, distCenter, offset)
	frame:ClearAllPoints()
	if distBottom < distTop and distBottom < distCenter then
		frame:SetPoint('BOTTOMRIGHT', otherFrame, 'BOTTOMLEFT', -offset, 0)
	elseif distTop < distBottom and distTop < distCenter then
		frame:SetPoint('TOPRIGHT', otherFrame, 'TOPLEFT', -offset, 0)
	else
		frame:SetPoint('RIGHT', otherFrame, 'LEFT', -offset, 0)
	end
	frame:SetSize(otherFrame:GetWidth(), otherFrame:GetHeight())
end

local function AttachToRight(frame, otherFrame, distTop, distBottom, distCenter, offset)
	frame:ClearAllPoints()
	if distBottom < distTop and distBottom < distCenter then
		frame:SetPoint('BOTTOMLEFT', otherFrame, 'BOTTOMRIGHT', offset, 0)
	elseif distTop < distBottom and distTop < distCenter then
		frame:SetPoint('TOPLEFT', otherFrame, 'TOPRIGHT', offset, 0)
	else
		frame:SetPoint('LEFT', otherFrame, 'RIGHT', offset, 0)
	end
	frame:SetSize(otherFrame:GetWidth(), otherFrame:GetHeight())
end

function StickFrames(frame, otherFrame, tolerance, xOff, yOff)
	local xOff, yOff = xOff or 0, yOff or 0
	local left = frame:GetLeft()
	local right = frame:GetRight()
	local top = frame:GetTop()
	local bottom = frame:GetBottom()
	local centerX, centerY = frame:GetCenter()

	if not CanAttach(frame, otherFrame) then return end
	
	if left and right and top and bottom and centerX then
		local oScale = otherFrame:GetScale()
		left = left / oScale
		right = right / oScale
		top = top / oScale
		bottom = bottom /oScale
		centerX = centerX / oScale
		centerY = centerY / oScale
	else return end

	local oLeft = otherFrame:GetLeft()
	local oRight = otherFrame:GetRight()
	local oTop = otherFrame:GetTop()
	local oBottom = otherFrame:GetBottom()
	local oCenterX, oCenterY = otherFrame:GetCenter()

	if oLeft and oRight and oTop and oBottom and oCenterX then
		local scale = frame:GetScale()
		oCenterX = oCenterX / scale
		oCenterY = oCenterY / scale
		oLeft = oLeft / scale
		oRight = oRight / scale
		oTop = oTop / scale
		oBottom = oBottom / scale
	else return end

	if(oLeft - tolerance <= left and oRight + tolerance >= right) or (left - tolerance <= oLeft and right + tolerance >= oRight)then
		local distCenter = math.abs(oCenterX - centerX)
		local distLeft = math.abs(oLeft - left)
		local distRight = math.abs(right - oRight)

		if math.abs(oTop - bottom) <= tolerance then 
			return AttachToTop(frame, otherFrame, distLeft, distRight, distCenter, yOff) 
		elseif math.abs(oBottom - top) <= tolerance then 
			return AttachToBottom(frame, otherFrame, distLeft, distRight, distCenter, yOff)
		end
	end

	if(oTop + tolerance >= top and oBottom - tolerance <= bottom) or (top + tolerance >= oTop and bottom - tolerance <= oBottom) then
		local distCenter = math.abs(oCenterY - centerY)
		local distTop = math.abs(oTop - top)
		local distBottom = math.abs(oBottom - bottom)
		
		if math.abs(oLeft - right) <= tolerance then return AttachToLeft(frame, otherFrame, distTop, distBottom, distCenter, xOff) end
		if math.abs(oRight - left) <= tolerance then return AttachToRight(frame, otherFrame, distTop, distBottom, distCenter, xOff) end
	end
end

-- end stick region


-- Mover addon by Allez

local moving = false
local movers = {}

local SetPosition = function(mover)
	local ap, _, rp, x, y = mover:GetPoint()
	SCDB.UISavedPositions[mover.frame:GetName()] = {ap, "UIParent", rp, ceil(x), ceil(y)}
end

local OnDragStart = function(self)
	self:StartMoving()
	self.frame:ClearAllPoints()
	self.frame:SetAllPoints(self)
end

local OnDragStop = function(self)
	self:StopMovingOrSizing()
	SetPosition(self)
end

local CreateMover = function(frame)
	local mover = CreateFrame("Frame", nil, UIParent)
	mover:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", })
	mover:SetBackdropColor(1, 1, 1, .4)
	mover:SetAllPoints(frame)
	mover:SetFrameStrata("TOOLTIP")
	mover:EnableMouse(true)
	mover:SetMovable(true)
	mover:RegisterForDrag("LeftButton")
	mover:SetScript("OnDragStart", OnDragStart)
	mover:SetScript("OnDragStop", OnDragStop)
	mover.frame = frame
	mover.name = mover:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	mover.name:SetFont(GameFontNormal:GetFont())
	mover.name:SetPoint("CENTER")
	mover.name:SetTextColor(1, 1, 1)
	mover.name:SetText(frame:GetName())
	movers[frame:GetName()] = mover
end

local GetMover = function(frame)
	if movers[frame:GetName()] then
		return movers[frame:GetName()]
	else
		return CreateMover(frame)
	end
end

StaticPopupDialogs["MOVE_UI"] = {
	text = "Reload UI to save changes?", 
	button1 = ACCEPT, 
	button2 = CANCEL,
	OnAccept = ReloadUI,
	timeout = 0, 
	whileDead = 1,
	hideOnEscape = 1, 
}

local InitMove = function(msg)
	if InCombatLockdown() then print(ERR_NOT_IN_COMBAT) return end
	if msg and msg == 'reset' then
		SCDB.UISavedPositions = {}
		ReloadUI()
		return
	end
	if SlashCmdList["BlizzCastbars"] then SlashCmdList["BlizzCastbars"]() end
	if SlashCmdList["EclipseBar"] then SlashCmdList["EclipseBar"]() end
	if not moving then
		for i, v in pairs(UIMovableFrames) do
			local mover = GetMover(v)
			if mover then mover:Show() end
		end
		moving = true
	else
		for i, v in pairs(movers) do
			v:Hide()
		end
		moving = false
		StaticPopup_Show("MOVE_UI")
	end
end

local RestoreUI = function(self)
	if InCombatLockdown() then
		if not self.shedule then self.shedule = CreateFrame("Frame", nil, self) end
		self.shedule:RegisterEvent("PLAYER_REGEN_ENABLED")
		self.shedule:SetScript("OnEvent", function(self)
			RestoreUI(self:GetParent())
			self:UnregisterEvent("PLAYER_REGEN_ENABLED")
			self:SetScript("OnEvent", nil)
		end)
		return
	end
	for frame_name, point in pairs(SCDB.UISavedPositions) do
		if _G[frame_name] then
			_G[frame_name]:ClearAllPoints()
			_G[frame_name]:SetPoint(unpack(point))
		end
	end
end

local frame = CreateFrame("Frame")
frame:RegisterEvent("PLAYER_ENTERING_WORLD")
frame:SetScript("OnEvent", function(self, event)
	self:UnregisterEvent(event)
	RestoreUI(self)
end)

SlashCmdList["MoveUI"] = InitMove
SLASH_MoveUI1 = "/mf"